-- Testlogik Praktikumsboard April 2005, Nov. 2006 A.Schnell
-- Rotary Encoder counter value is displayed as hex value on 7 segment display
-- clock_div(10) multiplexes displays
-- yellow LED shows direction of Rot.E ncoder
-- green LED brightness is modulated by Rot. Encoder counter value 
-- a simple PWM is used to define on/off ratio, clock_div(7) increments PWM_Accu 
-- Running light on LEDs with 0b00111111, clocked by clock_div(18)
-- keypad is not used here

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity prakt is
    Port ( clock : in std_logic;	  -- clock on board, 1.84 Mhz PIN 1 CPLD
	 		  load : in std_logic;	  -- pushbutton on board, PIN 33 CPLD
	 		  LEDout : out std_logic_vector(7 downto 0);	 -- for lauflicht
           led1 : out std_logic;	  -- pulse width by counter value
           led2 : out std_logic;	  -- direction of RE
			  re0 : in std_logic;  -- rotary (quadrature) encoder input
			  re1 : in std_logic;  -- dito
--           data0 : in std_logic;
--           data1 : in std_logic;
--           Din : inout std_logic_vector(7 downto 0);  -- DIL for Lauflicht Input
           anode0 : out std_logic;	  -- first 7 segment display
           anode1 : out std_logic;	  -- second 7 segment display
           d7segments : out std_logic_vector(7 downto 0)); -- 7 segment display
--           keyout : out std_logic_vector(2 downto 0);
--           keyin : in std_logic_vector(3 downto 0));
end prakt;

architecture Behavioral of prakt is
	signal clk_div: std_logic_vector(18 downto 0); -- counter for clock divider
  	signal SR_int: std_logic_vector(7 downto 0);	  -- shift register for lauflicht
	signal temp: std_logic_vector(3 downto 0);	  

	signal PWM_Accu: std_logic_vector(8 downto 0);

	type STATE_TYPE1 is (Reset,S00,S01,S11,S10); -- states for RotaryEncoder
	signal REState: STATE_TYPE1; 
	signal Quad: std_logic_vector(1 downto 0);  -- Latch for Inputs to RE
	signal counter: std_logic_vector(7 downto 0);

begin

	process(clk_div(7), counter)  -- simple PWM process, the on time of the LED
	begin									-- is defined by clk_div(7). off by the RE counter value
		if rising_edge(clk_div(7)) then
			PWM_Accu <= ("0" & PWM_Accu(7 downto 0)) + ("0" & counter);
		end if;
	end process;

	led1 <= PWM_Accu(8);

   process(clk_div(13),load,counter) -- Rotary Encoder State Machine ------------------------------------
		begin
			if load = '0' then  -- Initialisation
				REState <= Reset;
				counter <= "00000000";
				led2 <= '0'; -- dir led off
			elsif clk_div(13)'event and clk_div(13) = '1' then
				Quad(0) <= re0; 
				Quad(1) <= re1;  -- resync input lines

				case REState is -- State transitions
					 when Reset =>	 -- after Reset enter initial state
						if Quad = "00" then REState <= S00; end if;	
						if Quad = "01" then REState <= S01; end if;
						if Quad = "10" then REState <= S10; end if;
						if Quad = "11" then REState <= S11; end if;
					 when S00 =>
						if Quad = "00" then REState <= S00; end if;
						if Quad = "01" then REState <= S01; led2 <= '0'; counter <= counter +1; end if;
						if Quad = "10" then REState <= S10; led2 <= '1'; counter <= counter -1; end if;
						if Quad = "11" then REState <= Reset; end if; -- error
					 when S01 =>
						if Quad = "01" then REState <= S01; end if;
						if Quad = "11" then REState <= S11; led2 <= '0'; counter <= counter +1; end if;
						if Quad = "00" then REState <= S00; led2 <= '1'; counter <= counter -1; end if;
						if Quad = "10" then REState <= Reset; end if;
					 when S11 =>
						if Quad = "11" then REState <= S11; end if;
						if Quad = "10" then REState <= S10; led2 <= '0'; counter <= counter +1; end if;
						if Quad = "01" then REState <= S01; led2 <= '1'; counter <= counter -1; end if;
						if Quad = "00" then REState <= Reset; end if;
					 when S10 =>
						if Quad = "10" then REState <= S10; end if;
						if Quad = "00" then REState <= S00; led2 <= '0'; counter <= counter +1; end if;
						if Quad = "11" then REState <= S11; led2 <= '1'; counter <= counter -1; end if;
						if Quad = "01" then REState <= Reset; end if;
					 when others => REState <= Reset;
				end case;
			end if;	
	end process;


	anode0 <= clk_div(10); -- multiplexing display
	anode1 <= not clk_div(10);

	process(clk_div(10))
		begin
		if clk_div(10) = '1' then temp <= counter(3 downto 0); -- rechtes display
		else temp <= counter(7 downto 4);							 -- linkes display
		end if;

		case temp is					    --"abcdefg."	  0 -> led on										    
			 when "0000" => d7segments <= "10000001";	  -- 0	bcdefg  	  
		    when "0001" => d7segments <= "11101101";   -- 1	dg		  		 -c-
			 when "0010" => d7segments <= "01000011";	  -- 2	cdaef	  		b   d
			 when "0011" => d7segments <= "01001001";	  -- 3	cdagf			 -a-
			 when "0100" => d7segments <= "00101101";   -- 4	bdag	  		e   g
			 when "0101" => d7segments <= "00011001";	  -- 5	cbagf			 -f-   .
			 when "0110" => d7segments <= "00010001";   -- 6	cbaegf
			 when "0111" => d7segments <= "11001101";   -- 7	cdg
			 when "1000" => d7segments <= "00000001";	  -- 8	abcdefg
			 when "1001" => d7segments <= "00001001"; 	-- 9	abcdfg
			 when "1010" => d7segments <= "00000101";	   -- *	abcdeg  10	hex A
			 when "1011" => d7segments <= "00110001";	   -- b	bcdefg  11  hex b
			 when "1100" => d7segments <= "10010011";    -- #	cbef	  12	hex C
			 when "1101" => d7segments <= "01100001";	   -- d	bcdefg  13  hex d
			 when "1110" => d7segments <= "00010011";	   -- e	bcdefg  14  hex E
			 when "1111" => d7segments <= "00010111";	   -- f	bcdefg  15  hex F
			 when others => d7segments <= "11111111";		-- all off
		end case;
	end process;

	process(clk_div(18)) -- Lauflicht Prozess	------------------------------
	begin
		if clk_div(18)'event and clk_div(18) = '1' then
			if load = '0' then SR_int <= "00111111";	-- Datenuebernahme mit load Button 0 -> LED ON
			else SR_int <= SR_int(6 downto 0) & SR_int(7);  -- rotiert nach rechts
			end if;
		end if;
	end process;

	LEDout <= SR_int;


	process (clock) -- clock prescaler -------------------------------------
	begin
		if (clock'event and clock = '1') then
		  clk_div <=  clk_div + 1;  -- counter
		end if;
	end process;



end Behavioral;
