-- scancode von ps2 lesen,auf 7segment ausgeben  Okt. 2004 A.Schnell
-- April 2005 Mod fuer CPLD Board XC9572xl

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ps2 is
    Port ( kdata : in std_logic;		   -- ps2 anschluesse
           kclock : in std_logic;
           anode : out std_logic_vector(1 downto 0);
           ssegments : out std_logic_vector(7 downto 0);

			  reset : in std_logic;
           clock : in std_logic);
end ps2;

architecture Behavioral of ps2 is
	signal clk_div : std_logic_vector (12 downto 0);  -- clock divider clock:1.84MHz
	signal selector : std_logic_vector(1 downto 0);	-- selector for 7segment mux
	signal pclk : std_logic;	-- clock for ps2 flipflops
	signal kdata1, kdata2 : std_logic; 	-- ps2 data input ff
	signal kclock1, kclock2 : std_logic;	--	ps2 clock input ff
	signal shiftreg1 : std_logic_vector(10 downto 0);	-- for ps2 databits
	signal shiftreg2 : std_logic_vector(10 downto 0);	-- for ps2 databits
	signal muxout : std_logic_vector(3 downto 0); -- 4 bit mux for 7segmentdaten
	signal scancode1 : std_logic_vector(7 downto 0); -- from ps2
	signal scancode2 : std_logic_vector(7 downto 0); -- from ps2 for mouse

	subtype counter is std_logic_vector(9 downto 0); -- def. of 10 Bit counter (0..1024)

begin

	selector(0) <= clk_div(9); 	-- 7 segment selector, in prakt board only 2 Displays
	selector(1) <= clk_div(10);	-- available, only selector(0) is used
	pclk <= clock;				-- clock for resynking PS2 signals: 1.84MHz


	process (pclk, reset, kclock, kdata)  -- structuring ps2 signals
	begin
		if(reset = '0') then	-- reset flipflops 
		   kdata1 <= '0'; kdata2 <= '0'; kclock1 <= '0'; kclock2 <= '0';
	   else
		   if (pclk'event) and (pclk = '1') then
			   kdata2 <= kdata1; kdata1 <= kdata; 	-- resynking to system clock
				kclock2 <= kclock1; kclock1 <= kclock;
			end if;
		end if;
	end process;

	process( kdata2, kclock2, reset)	-- shiftregisters to clock scan codes in
	begin
		if( reset = '0') then
			shiftreg1 <= "00000000000"; shiftreg2 <= "00000000000";
		else
			if(kclock2 = '0' and kclock2'event) then
			  	shiftreg1(10 downto 0) <= kdata2 & shiftreg1(10 downto 1);
				shiftreg2(10 downto 0) <= shiftreg1(0) & shiftreg2(10 downto 1);
			end if;
		end if;
	end process;

	process(shiftreg1, shiftreg2, reset, kclock2) -- uebernahme in scancoderegister
	begin
		if(reset = '0') then
			scancode1 <= "00000000"; scancode2 <= "00000000";
		else
			if(kclock2'event and kclock2 = '1' and shiftreg2(8 downto 1) = "11110000") 
			   then scancode1 <= shiftreg1(8 downto 1);
			end if;
		end if;			
	end process;

	muxout <= scancode1(7 downto 4) when selector(0) = '0' else	 -- only 2 displays 
				 scancode1(3 downto 0) when selector(0) = '1' else  -- active
				 "0000";
	anode <= "10" when selector(0) = '0' else
	         "01" when selector(0) = '1' else
				"11";
	 -- " left .. right " display
	        ssegments <= "11000000" when muxout = "0000" else	  -- 0
					 "11111001" when muxout = "0001" else	  -- 1
	 			    "10100100" when muxout = "0010" else	  -- 2
					 "10110000" when muxout = "0011" else	  -- 3
					 "10011001" when muxout = "0100" else	  -- 4
					 "10010010" when muxout = "0101" else	  -- 5
					 "10000010" when muxout = "0110" else   -- 6
					 "11111000" when muxout = "0111" else   -- 7
					 "10000000" when muxout = "1000" else   -- 8
					 "10010000" when muxout = "1001" else   -- 9
					 "10001000" when muxout = "1010" else   -- A
					 "10000011" when muxout = "1011" else   -- b
					 "11000110" when muxout = "1100" else   -- C
					 "10100001" when muxout = "1101" else   -- d
					 "10000110" when muxout = "1110" else   -- E
					 "10001110" when muxout = "1111" else   -- F
					 "11111111";
		         -- " dp g f e d c b a "

   process(clock)	  -- clock divider chain
   begin
      if (clock'event) and (clock = '1') then
	   clk_div <= clk_div + 1;
      end if;
	end process;

end Behavioral;
